#!/usr/bin/perl -w

######################################################
#
#	2Dvs3D wrapper exploration
#
#	The 2Dvs3D wrapper has the ability to produce a 2D and 3D exploration
#	by doing a sweep on the x-axis and y-axis for 2 input parameters
#	and show the iso curvers of various values of the performance parameter
#
#	The wrapper needs as configurations inputs the 2D dc and server configuration
#   to calculate the targeted TCO and the 3D dc and server configuration for
#   the design space exploration
#
#	The results are printed in a file <output_file_name>.txt
#	and a graph can be plot in a file <output_file_name>.eps
#
#	How to use it:
#	./2Dvs3D.pl	        <2D dc config file>								datacenter configuration file
#						<2D resource config file>						resource configuration file
#						<3D dc config file>								datacenter configuration file
#						<3D resource config file>						resource configuration file
#						<output_file_name>								filename of the output file without extension (the extensions are .txt and .eps)
#						<x_parameter_name>								parameter x name 
#						<x_parameter_value_ref>							reference value of x 
#						<y_parameter_name>								parameter y name 
#						<y_parameter_value_ref>							reference value of y 
#
#																
#
#  Example of command:
# ./2Dvs3D.pl ../../input_example/ISPASS/dc.LPO.params ../../input_example/ISPASS/LPO.params ../../input_example/ISPASS/dc.LPO.params ../../input_example/ISPASS/LPO3D.params 2Dvs3D c_chip 100 p_chip 5.5
#
######################################################


use strict;

use POSIX qw/ceil/;

use File::Copy;

use lib "../lib/"; 
use helper;

use Data::Dumper;

######################################################
#
# Get input parameters
#
######################################################

my $argv_size = scalar @ARGV;
if ($argv_size < 8)
{
	print "usage: ./2Dvs3D.pl <dc config file> <resource config file> <output_file_name> <x_parameter_name> <x_parameter_value_ref> <y_parameter_name> <y_parameter_value_ref> \n";
	exit(0);
}

my $dc2D_filename = shift;
my $resource2D_filename = shift;

my $dc_filename = shift;
my $resource_filename = shift;

my $filename =  shift;

my $x_name = shift;
my $x_ref = shift;

my $y_name = shift;
my $y_ref = shift;

######################################################
#
# General Initialization
#
######################################################

#create the helper
my $helper = helper->new("");

#check that the inputs are correct
if($helper->checkInputParameterName($x_name)==-1){print("Execution stopped\n");exit(0); }
if($helper->checkInputParameterName($y_name)==-1){print("Execution stopped\n");exit(0); }

#to get the current dir
my $currentdir=`pwd`;
chomp $currentdir;

system("cp $dc2D_filename template/dc.2D");
system("cp $resource2D_filename template/server.2D");
#call the eetco kernel and keep the output
$helper->callKernel("$currentdir/template/dc.2D","$currentdir/template/server.2D","$currentdir/output/2D_TCO");

#Calculate the 2D TCO
my $ptr_res2D=$helper->parserResultKernel("$currentdir/output/2D_TCO");
my %res2D=%{$ptr_res2D};
my $tco_2D = $res2D{"tco"};

system("rm template/dc.2D");
system("rm template/server.2D");

#open outputfile
open(OUTPUTFILE,">$currentdir/$filename.txt") || die ("Error when opening $currentdir/$filename.txt") ;


######################################################
#
# Specific Initialization
#
######################################################

#get the parameters from files
my $ptr_dc=$helper->parserDCFile("$dc_filename");
my %dc=%{$ptr_dc};

my $ptr_resource=$helper->parserResourceFile("$resource_filename");
my %resource=%{$ptr_resource};

####################################################
# main exploration loop
####################################################

my $perf_min = 1;
my $perf_max = 2;
my $perf_step = 0.2;

print(OUTPUTFILE "Chip cost\t Chip power\n");
for(my $performance = $perf_min; $performance <= $perf_max; $performance += $perf_step)
{
	print(OUTPUTFILE "\tperf=${performance}x");
}
print(OUTPUTFILE "\n");

my $init_performance = $resource{"n_srvmodulesreq"};

my $count_inter = 0;

#exploration loop of peformance
for(my $performance = $perf_min; $performance <= $perf_max; $performance += $perf_step)
{
	#generate the current configuration file
	$helper->createParameterFile("$currentdir/dc.params",\%dc);
	
	my $current_performance = ceil($init_performance/$performance);
	$helper->createParameterFile("$currentdir/s.params",\%resource);

	#call the eetco kernel and keep the output
	$helper->callKernel("$currentdir/dc.params","$currentdir/s.params","$currentdir/output/perf_$performance.txt");

	system("../iso/iso.pl ../2Dvs3D/dc.params s.params chip_$performance tco:$tco_2D 0.001 n_srvmodulesreq $init_performance $current_performance $current_performance 1 absolute $x_name $x_ref nognuplot");
	#get the result value
	system("mv chip_$performance.txt output/chip_$performance.txt");
 	open(FILE, "output/chip_$performance.txt") or die("Unable to open file");
 	my @data = <FILE>; 
	close(FILE);
	my @line = split('\t', $data[0]);
	my $x = $line[1];

	#generate the current configuration file
	$helper->createParameterFile("$currentdir/dc.params",\%dc);
	$helper->createParameterFile("$currentdir/s.params",\%resource);

	#call the eetco kernel and keep the output
	$helper->callKernel("$currentdir/dc.params","$currentdir/s.params","$currentdir/output/perf_$performance.txt");

	system("../iso/iso.pl dc.params s.params power_$performance tco:$tco_2D 0.001 n_srvmodulesreq $init_performance $current_performance $current_performance 1 absolute $y_name $y_ref nognuplot");
	#get the result value
	system("mv power_$performance.txt output/power_$performance.txt");
 	open(FILE, "output/power_$performance.txt") or die("Unable to open file");
 	@data = <FILE>; 
	close(FILE);
	@line = split('\t', $data[0]);
	my $y = $line[1];
		
	$count_inter++;
	print(OUTPUTFILE "$x");
	for(my $i = 0; $i < $count_inter; $i++)
	{
		print(OUTPUTFILE "\t");
	}
	print(OUTPUTFILE "1\n");

	print(OUTPUTFILE "1");
	for(my $i = 0; $i < $count_inter; $i++)
	{
		print(OUTPUTFILE "\t");
	}
	print(OUTPUTFILE "$y\n");
}

close(OUTPUTFILE);

####################################################
# clean
####################################################
$helper->deleteFile("$currentdir/dc.params");
$helper->deleteFile("$currentdir/s.params");
$helper->deleteFileInDir("$currentdir/output/");

